/*
 * MultiMail offline mail reader
 * area list

 Copyright (c) 1996 Kolossvary Tamas <thomas@vma.bme.hu>
 Copyright (c) 1998 William McBrine <wmcbrine@clark.net>

 Distributed under the GNU General Public License.
 For details, see the file COPYING in the parent directory. */

#include "interfac.h"
#include "isoconv.h"

//**************        LittleAreaListWindow    *****************

LittleAreaListWindow::LittleAreaListWindow(char param_key)
{
	int dest;

	disp = 0;
	do
		mm.areaList->gotoArea(disp++);
	while (mm.areaList->isCollection());
	disp--;

	key = param_key;
	list_max_y = LINES - 10;
	list = new InfoWin(LINES - 6, 25, 2, (COLS - 25) / 2, C_LALBTEXT,
		NULL, 0, 4, 3);
	dest = mm.letterList->getAreaID() - disp;
	while (active != dest)
		Move(DOWN);
	list->putstring(1, 2, "Reply goes to area:");
	Draw();
}

LittleAreaListWindow::~LittleAreaListWindow()
{
	mm.areaList->gotoArea(active + disp);
	interface->letterwindow.set_Letter_Params(active + disp, key);
	delete list;
}

int LittleAreaListWindow::NumOfItems()
{
	return mm.areaList->noOfAreas() - disp;
}

void LittleAreaListWindow::oneLine(int i)
{
	mm.areaList->gotoArea(position + i + disp);
	sprintf(list->lineBuf, " %-21.21s ", mm.areaList->getDescription());
	list->oneline(i, (position + i == active) ? A_STANDOUT : C_LALLINES);
}

void LittleAreaListWindow::extrakeys(int key)
{
	if (key == 'C')
		interface->TransToggle();
}

//*************         AreaListWindow          ******************

void AreaListWindow::ReDraw()
{
	list->touch();
	info->touch();
}

void AreaListWindow::FirstUnread()
{
	int i;

	oldPos = oldActive = -100;
	position = active = 0;
	list_max_y = LINES - 15;
	for (i = 0; i < NumOfItems(); i++) {
		mm.areaList->gotoActive(i);
		if (!mm.areaList->getNoOfUnread())
			Move(DOWN);
		else
			break;
	}
	Draw();
}

int AreaListWindow::NumOfItems()
{
	return mm.areaList->noOfActive();
}

void AreaListWindow::oneLine(int i)
{
	char *p = list->lineBuf;

	mm.areaList->gotoActive(position + i);

	if (position + i == active) {
		sprintf(p, "%-20.20s", mm.areaList->getAreaType());
		info->putstring(2, 8, p);
		sprintf(p, format2, mm.areaList->getDescription());
		charconv_in(p);
		info->putstring(3, 20, p);
		info->delay_update();
	}
	p += sprintf(p, format, mm.areaList->getShortName(),
		mm.areaList->getDescription());
	if (mm.areaList->getNoOfLetters())
		p += sprintf(p, "  %5d  ", mm.areaList->getNoOfLetters());
	else
		p += sprintf(p, "      .  ");
	if (mm.areaList->getNoOfUnread())
		p += sprintf(p, "  %5d   ", mm.areaList->getNoOfUnread());
	else
		p += sprintf(p, "      .   ");
	if (mode)
		if (mm.areaList->getNoOfPersonal())
			sprintf(p, "   %5d   ",
				mm.areaList->getNoOfPersonal());
		else
			sprintf(p, "       .   ");

	list->oneline(i, (position + i == active) ? A_STANDOUT : 
		((mm.areaList->isReplyArea() ? C_ALREPLINE : C_ALPACKETLINE) |
		((mm.areaList->getNoOfUnread() > 0) ? A_BOLD : 0)));
}

void AreaListWindow::Select()
{
	mm.areaList->gotoActive(active);
}

void AreaListWindow::ResetActive()
{
	active = mm.areaList->getActive();
}

void AreaListWindow::MakeActive()
{
	int padding;
	char tmp[25];

	oldPos = oldActive = -100;
	mode = mm.driverList->hasPersonal();

	sprintf(tmp, "Message Areas (%d)", NumOfItems());

	list_max_y = LINES - 15;
	list = new InfoWin(LINES - 12, COLS - 4, 2, 2, C_ALBORDER,
		tmp, C_ALBTEXT);

	list->attrib(C_ALHEADTEXT);
	list->putstring(1, 3, "Area#  Description");
	int newloc = COLS - 21;
	if (mode)
		newloc -= 11;
	list->putstring(1, newloc, "Total   Unread");
	if (mode)
		list->putstring(1, COLS - 15, "Personal");

	info = new ShadowedWin(5, COLS - 4, LINES - 11, 2, C_ALBORDER,
		"Info On The Specified Area", C_ALBTEXT);

	info->putch(0, 0, ACS_LTEE);
	info->putch(0, COLS - 5, ACS_RTEE);

	padding = COLS - 34;
	if (mode)
		padding -= 11;
	sprintf(format, " %%6s  %%-%d.%ds", padding, padding);

	info->attrib(C_ALINFOTEXT);
	info->putstring(1, 2, " BBS:                        Sysop: ");
	info->putstring(2, 2, "Type:                       Packet: ");
	info->putstring(3, 2, "Area description: ");

	info->attrib(C_ALINFOTEXT2);
	sprintf(tmp, "%-20.20s", mm.resourceObject->get(BBSName));
	charconv_in(tmp);
	info->putstring(1, 8, tmp);

	sprintf(tmp, "%-20.20s", mm.resourceObject->get(SysOpName));
	charconv_in(tmp);
	info->putstring(1, 38, tmp);

	info->putstring(2, 38, mm.resourceObject->get(PacketName));

	padding = COLS - 26;
	sprintf(format2, "%%-%d.%ds", padding, padding);

	Draw();
	Select();
}

void AreaListWindow::Delete()
{
	delete info;
	delete list;
}

void AreaListWindow::PrevAvail()
{
	do {
		Move(UP);
		Select();
	} while (!mm.areaList->getNoOfLetters() &&
		 (mm.areaList->getActive() > 0));
	Draw();
}

void AreaListWindow::NextAvail()
{
	do {
		Move(DOWN);
		Select();
	} while (!mm.areaList->getNoOfLetters() &&
		 (mm.areaList->getActive() <
		  mm.areaList->noOfActive() - 1));
	Draw();
}

void AreaListWindow::extrakeys(int key)
{
	Select();
	switch (key) {
	case 'A':
		interface->addressbook(1);
		break;
	case 'C':
		interface->TransToggle();
		break;
	case 'E':
		if (!mm.areaList->isCollection()) {
			if (mm.areaList->isNetmail()) {
				interface->addressbook();
				Select();
			}
			interface->letterwindow.set_Letter_Params(
				mm.areaList->getAreaNo(), 'E');
			interface->letterwindow.EnterLetter();
			interface->changestate(arealist);
		}
		break;
	case '-':
	case KEY_LEFT:
		PrevAvail();
		break;
	case '+':
	case KEY_RIGHT:
		NextAvail();
		break;
	case '!':
	case KEY_F(2):
		if (interface->WarningWindow(
	"This will overwrite any existing reply packet. Continue?",
			  "Yes", "No"))
			interface->create_reply_packet();
		ReDraw();
		break;
	case 'L':
		mm.areaList->relist();
		ResetActive();
		interface->changestate(arealist);
	}
}
