/*
 * MultiMail offline mail reader
 * help windows

 Copyright (c) 1996 Kolossvary Tamas <thomas@vma.bme.hu>
 Copyright (c) 1998 William McBrine <wmcbrine@clark.net>

 Distributed under the GNU General Public License.
 For details, see the file COPYING in the parent directory. */

#include "interfac.h"

void HelpWindow::newHelpMenu(const char **keys, const char **func)
{
	int x, z;

	menu = new Win(3, COLS - 2, LINES - 4, 1, C_HELP2 | A_BOLD);

	midpos = (COLS / 2) - 6;
	endpos = COLS - 21;

	for (x = 0; x < 3; x++) {
		z = x * 3;
		if (keys[z])
			menu->putstring(x, 1, keys[z]);
		if (keys[++z])
			menu->putstring(x, midpos - strlen(keys[z]), keys[z]);
		if (keys[++z])
			menu->putstring(x, endpos - strlen(keys[z]), keys[z]);
	}

	menu->attrib(C_HELP1 | A_BOLD);

	for (x = 0; x < 3; x++) {
		z = x * 3;
		if (func[z])
			menu->putstring(x, 2, func[z]);
		if (func[++z])
			menu->putstring(x, midpos, func[z]);
		if (func[++z])
			menu->putstring(x, endpos, func[z]);
	}
}

void HelpWindow::h_packetlist()
{
	static const char *keys[] = {
		"Q", "Enter", "A",
		"K", "Space, F", "B",
		0, "+", "-"
	}, *func[] = {
		": Quit", ": Select packet", ": Addressbook",
		": Kill packet", ": aliases for PgDn", ": alias for PgUp",
		0, ": alias for Right", ": alias for Left"
	};

	newHelpMenu(keys, func);
	menu->delay_update();
}

void HelpWindow::h_arealist()
{
	static const char *keys[] = {
		"Q", "Enter", "A",
		"E", "Right", "Left",
		"L", 0, 0
	}, *func[] = {
		": Back to packet list", ": Select area", ": Addressbook",
		": Enter letter in area", ": Next non-empty",
		": Prev non-empty", ": Long/short area list", 0, 0
	};

	newHelpMenu(keys, func);

	mm.areaList->gotoArea(REPLY_AREA);
	if (mm.areaList->getNoOfLetters() > 0) {
		menu->putstring(2, midpos, ": Make reply packet");
		menu->attrib(C_HELP2 | A_BOLD);
		menu->putstring(2, midpos - 5, "F2, !");
	}
	menu->delay_update();
}

void HelpWindow::h_letterlist()
{
	static const char *keys[] = {
		"L", "Enter", "A",
		"E", "Right", "Left",
		"S", "U", "M"
	}, *func[] = {
		": List all/unread", ": Read letter", ": Addressbook",
		": Enter letter in area", ": Next unread",
		": Previous unread", ": Save all in area",
		": Unread/read toggle", ": Mark/unmark"
	}, *repkeys[] = {
		"K", "Enter", "A",
		"E", 0, 0,
		"K", 0, 0
	}, *repfunc[] = {
		": Kill letter", ": Read letter", ": Addressbook",
		": Edit letter", 0, 0,
		": Save all in area", 0, 0
	};

	if (!mm.areaList->isReplyArea())
		newHelpMenu(keys, func);
	else
		newHelpMenu(repkeys, repfunc);

	menu->delay_update();
}

void HelpWindow::h_letter()
{
	static const char *common[] = {
		"S - Save letter",
		"A - Addressbook",
		"C - toggle Character set translation",
		"V, ^V, ^A - ANSI viewer",
		"Left - Previous letter",
		"Right, Enter - Next letter",
		"Space - Page through area",
		"Q - Go back to index",
		"",
		"Plus the standard up/down and PgUp/PgDn keys"
	}, *regular[] = {
		"E - Enter letter",
		"R - Reply to letter",
		"O - reply to Original sender",
		"N - Netmail reply",
		"T - Take tagline",
		"M - Mark/unmark letter",
		"U - Unread/read toggle"
	}, *reply[] = {
		"K - Kill letter",
		"E - Edit letter"
	};

	int base = mm.areaList->isReplyArea() ? 2 : 7;

	menu = new ShadowedWin(base + 12, 50, ((LINES - 12) - base) / 2,
		COLS / 2 - 25, C_HELP3 | A_BOLD);
	menu->attrib(C_HELP4 | A_BOLD);

	const char **basechar = (base == 7) ? regular : reply;

	for (int x = 0; x < base; x++)
		menu->putstring(x + 1, 2, basechar[x]);

	for (int x = 0; x < 10; x++)
		menu->putstring(base + x + 1, 2, common[x]);

	menu->wtouch();
}

void HelpWindow::MakeActive()
{
	switch (interface->active()) {
	case letter_help:
		h_letter();
		break;
	case letterlist:
		h_letterlist();
		break;
	case arealist:
		h_arealist();
		break;
	case packetlist:
		h_packetlist();
	default:;
	}
}

void HelpWindow::Delete()
{
	switch (interface->active()) {
	case letter_help:
		delete (ShadowedWin *) menu;
		break;
	case letterlist:
	case arealist:
	case packetlist:
		delete menu;
	default:;
	}
}
