/*
 * MultiMail offline mail reader
 * most class definitions for the interface

 Copyright (c) 1996 Kolossvary Tamas <thomas@tvnet.hu>
 Copyright (c) 1997 John Zero <john@graphisoft.hu>
 Copyright (c) 1998 William McBrine <wmcbrine@clark.net>

 Distributed under the GNU General Public License.
 For details, see the file COPYING in the parent directory. */

#ifndef INTERFACE_H
#define INTERFACE_H

#define USE_SHADOWS	// "Shadowed" windows

#include "../mmail/mmail.h"

extern "C" {
#include <signal.h>
#include CURS_INC
}

#include "mmcolor.h"

enum direction {UP, DOWN, PGUP, PGDN, HOME, END};
enum statetype {packetlist, arealist, threadlist, letterlist, letter, 
		letter_help, littlearealist, address, tagwin, ansiwin};

#ifdef SIGWINCH
extern void sigwinchHandler(int);
#endif

extern int isoconvert;	// ISO8859-1 <--> CP437 (DOS) conversion?

#define TAGLINE_LENGTH 76

class tagline
{
 public:
 	tagline();
 	tagline *next;
 	char text[TAGLINE_LENGTH + 1];
};

class Win
{
 protected:
	WINDOW *win;
 public:
	Win(int, int, int, int, chtype);
	~Win();
	void Clear(chtype);
	void putch(int, int, chtype);
	void putchnstr(int, int, const chtype *, int);
	void putstring(int, int, const char *);
	void attrib(chtype);
	void horizline(int, int);
	void update();
	void delay_update();
	void wtouch();
	void wscroll(int);
	void cursor_on();
	void cursor_off();
	int keypressed();
	int inkey();
	void boxtitle(chtype, const char *, chtype);
};

class ShadowedWin : public Win
{
#ifdef USE_SHADOWS
	WINDOW *shadow;
#endif
 public:
	ShadowedWin(int, int, int, int, chtype, const char * = NULL,
		chtype = 0);
	~ShadowedWin();
	void touch();
	int getstring(int, int, char *, int, int, int);
};

class InfoWin : public ShadowedWin
{
	Win *info;
 public:
	char *lineBuf;

	InfoWin(int, int, int, int, chtype, const char * = NULL,
		chtype = 0, int = 3, int = 2);
	~InfoWin();
	void irefresh();
	void touch();
	void oneline(int, chtype);
	void iscrl(int);
};

class ListWindow
{
 protected:
	InfoWin *list;
  	int list_max_y, list_max_x;
	int position;	//the first element in the window
	int active;	//this is the highlited	
	int oldPos;	//position at last Draw()
	int oldActive;	//active at last Draw()

	void checkPos(int);
	void Draw();		//refreshes the window
	virtual int NumOfItems() = 0;
	virtual void oneLine(int) = 0;
	virtual void extrakeys(int) = 0;
 public:
 	ListWindow();
	void Move(direction);	//scrolloz
	virtual void ReDraw();	//apply touchwin on the windoz
	void KeyHandle(int);
};

class Welcome
{
 	ShadowedWin *window;
 public:
 	void MakeActive();
 	void Delete();
};

class Person
{
 public:
 	Person *next;
 	char name[30];
 	net_address netmail_addr;

	Person();
};

class AddressBook : public ListWindow
{
	Person *highlighted, **people;
	int NumOfPersons, NoEnter, inletter;

  	int NumOfItems();
	void oneLine(int);
	void extrakeys(int);

	void AddressFromLetter();
 	void ReadFile();
 	void DestroyChain();
 	void SetLetterThings();
 public:
	AddressBook();
	void MakeActive(int);
	void Delete();
};

class TaglineWindow : public ListWindow
{
  	int nodraw, NumOfTaglines;
	tagline head;

 	int NumOfItems();
	void oneLine(int);
	void extrakeys(int);

	void ReadFile();
	void DestroyChain();
	void RandomTagline();
  	const char *getTextOf(int); 
 public:
 	TaglineWindow();
	virtual ~TaglineWindow();
	void MakeActive();
	void Delete();
 	void EnterTagline(const char * = NULL);
};

class LittleAreaListWindow : public ListWindow
{
 	char key;
	int disp;

	int NumOfItems();
	void oneLine(int);
	void extrakeys(int);
 public:
 	LittleAreaListWindow(char);
 	virtual ~LittleAreaListWindow();
};

class PacketListWindow : public ListWindow
{
	Welcome welcome;
	file_list *packetList;

	int NumOfItems();
	void oneLine(int);
	void extrakeys(int);

	void ReList();
 public:
	PacketListWindow();
	virtual ~PacketListWindow();
 	void MakeActive();
 	void Delete();
	void Select();
	void OpenPacket();
};

class AreaListWindow : public ListWindow
{
 	ShadowedWin *info;
	char format[40], format2[40];
	int mode;

 	int NumOfItems();
	void oneLine(int);
	void extrakeys(int);

	void PrevAvail();
	void NextAvail();
 public:
	void ResetActive();
 	void MakeActive();
 	void Delete();
	void Select();
	void ReDraw();
	void FirstUnread();
};

class LetterListWindow : public ListWindow
{
	char format[50];

	int NumOfItems();
	void oneLine(int);
	void extrakeys(int);

	void NextUnread();
	void PrevUnread();
 public:
	void ResetActive();
	void MakeActive();
	void Delete();
	void Select();
	void FirstUnread();
};

class Line
{
 public:
	Line *next;
	char *text;
	unsigned length;

 	Line(int);
	~Line();
};

class LetterWindow
{
	Win *headbar, *header, *text, *statbar;
	Line **linelist;
	char key, tagline1[82], To[30];
	int letter_in_chain;	//0 = no letter in chain
	int position;		//which row is the first in the text window
	int NumOfLines;
	int dont_inc_pos;
	int y;			//height of the window, set by MakeActive
	int replyto_area;
	net_address NM;

	void lineCount();
	void oneLine(int);
	void Move(int);
	void netAdd(char *);
	int HeaderLine(ShadowedWin *, char *, int, int, int);
	int EnterHeader(char *, char *, char *, int *);
	void QuoteText(const char *);
	void DestroyChain();
	void editletter(const char *);
	void EditLetter();
	long reconvert(const char *);
	void write_to_file(FILE *);
	void GetTagline();
	void Previous();
	void NextDown();
 public:
	LetterWindow();
 	void MakeActive(int);
 	void Delete();
	void MakeChain(int);
	void Next();
 	void Draw();
 	void DrawHeader();
 	void DrawBody();
	void ReDraw();
	void Save(int);
	void EnterLetter();
	void StatToggle(int);
	net_address PickNetAddr();
	void set_Letter_Params(int, char);
	void set_Letter_Params(net_address *, const char *);
	void set_Tagline(const char *);
	void KeyHandle(int);
};

class HelpWindow
{
 	Win *menu;
	int midpos, endpos;

	void newHelpMenu(const char **, const char **);
	void h_packetlist();
	void h_arealist();
	void h_letterlist();
	void h_letter();
 public:
	void MakeActive();
	void Delete();
};

class AnsiLine
{
 private:
	AnsiLine *prev, *next;
 public:
	AnsiLine *getprev();
	AnsiLine *getnext(int = 0);
	chtype *text;
	unsigned length;
 	AnsiLine(int = 0, AnsiLine * = NULL);
	~AnsiLine();
};

class AnsiWindow
{
	int colorsused[64];
	char escparm[256];	//temp copy of ESC sequence parameters
	const char *title;
	const unsigned char *source;
	Win *header, *text, *statbar, *animtext;
	AnsiLine *head, *curr, **linelist;
	int position;		//which row is the first in the window
	int NumOfLines;
	int x, y;		//dimensions of the window
	int cpx, cpy, lpy;	//ANSI positions
	int spx, spy;		//stored ANSI positions
	int ccf, ccb, cfl, cbr, crv;	//colors and attributes
	int oldcolorx, oldcolory;
	int anim;		//animate mode?
	int ansiAbort;
	chtype attrib;		//current attribute

	void oneLine(int);
	void DrawBody();
	int getparm();
	void colreset();
	void colorset();
	const unsigned char *escfig(const unsigned char *);
	void posreset();
	void checkpos();
	void update(unsigned char);
	void ResetChain();
	void MakeChain(const unsigned char *);
	void DestroyChain();
	void animate();
	void statupdate(const char * = NULL);
 public:
	void set(const char *, const char *);
	void MakeActive();
	void Delete();
	void KeyHandle(int);
};

class Interface
{
	PacketListWindow packets;
	AddressBook addresses;
	HelpWindow helpwindow;
	LittleAreaListWindow *littleareas;

	Win *screen;
	ListWindow *currList;
 	statetype state, prevstate;
	char Key;
	int unsaved_reply, any_read;

	void init_colors();
	void oldstate(statetype);
 	int select();
 public:
	AreaListWindow areas;
	LetterListWindow letters;
	LetterWindow letterwindow;
	TaglineWindow taglines;
	AnsiWindow ansiwindow;

 	Interface();
	~Interface();
	void screen_init();
	int WarningWindow(const char *, const char *, const char *);
	void newstate(statetype);
	void changestate(statetype);
 	int back();		//returns 1 if we have to quit the program
	statetype active();
#ifdef SIGWINCH
	void sigwinch();
#endif
	void addressbook(int = 0);
	void Tagwin();
	void ansiLoop(const char *, const char *);
	void TransToggle();
	void kill_letter();
	void create_reply_packet();
	void setUnsaved();
	void setAnyRead();
	void KeyHandle();
};

extern mmail mm;
extern Interface *interface;

#endif
