/*
 * MultiMail offline mail reader
 * message list

 Copyright (c) 1996 Kolossvary Tamas <thomas@tvnet.hu>
 Copyright (c) 1997 John Zero <john@graphisoft.hu>
 Copyright (c) 1998 William McBrine <wmcbrine@clark.net>

 Distributed under the GNU General Public License.
 For details, see the file COPYING in the parent directory. */

#include "interfac.h"
#include "isoconv.h"
#include "mysystem.h"

void LetterListWindow::NextUnread()
{
	do {
		Move(DOWN);
		mm.letterList->gotoActive(active);
	} while (mm.letterList->getRead() && ((active + 1) < NumOfItems()));
	Draw();
}

void LetterListWindow::FirstUnread()
{
	position = 0;
	active = -1;
	NextUnread();
}

void LetterListWindow::PrevUnread()
{
	do {
		Move(UP);
		mm.letterList->gotoActive(active);
	} while (mm.letterList->getRead() && (active > 0));
	Draw();
}

int LetterListWindow::NumOfItems()
{
	return mm.letterList->noOfActive();
}

void LetterListWindow::oneLine(int i)
{
	mm.letterList->gotoActive(position + i);

	int st = mm.letterList->getStatus();

	char *p = list->lineBuf;
	p += sprintf(p, format, (st & MS_MARKED) ? 'M' : ' ',
		(st & MS_REPLIED) ? '~' : ' ', (st & MS_READ) ? '*' : ' ',
			mm.letterList->getMsgNum(), mm.letterList->getFrom(),
				mm.letterList->getTo(),
					mm.letterList->getSubject());

	if (mm.areaList->isCollection()) {
		const char *origArea = mm.areaList->
			getDescription(mm.letterList->getAreaID());
		if (origArea)
			sprintf(p - 15, " %-13.13s ", origArea);
	}

	list->oneline(i, (position + i == active) ? A_STANDOUT :
		(C_LISTWIN | ((st & MS_READ) ? 0 : A_BOLD)));
}

void LetterListWindow::MakeActive()
{
	char topline[72];
	int tot, maxFromLen, maxToLen, maxSubjLen;

	oldPos = oldActive = -100;
	tot = COLS - 19;
	maxSubjLen = tot / 2;
	tot -= maxSubjLen;
	maxToLen = tot / 2;
	maxFromLen = tot - maxToLen;

	sprintf(format, "%%c%%c%%c%%6d  %%-%d.%ds %%-%d.%ds %%-%d.%ds",
		maxFromLen, maxFromLen, maxToLen, maxToLen,
			maxSubjLen, maxSubjLen);

	interface->areas.Select();

	sprintf(topline, "Letters in %.60s", mm.areaList->getDescription());
	charconv_in(topline);

	list_max_y = (NumOfItems() < LINES - 11) ? NumOfItems() : LINES - 11;
	list = new InfoWin(list_max_y + 3, COLS - 4, 2, 2, C_LLBBORD,
		topline, C_LLTOPTEXT2);

	list->attrib(C_LLTOPTEXT1);
	list->putstring(0, 3, "Letters in ");

	sprintf(topline, "   Msg#  %%-%d.%ds %%-%d.%ds %%-%d.%ds",
		maxFromLen, maxFromLen, maxToLen, maxToLen,
			maxSubjLen, maxSubjLen);
	list->attrib(C_LLHEAD);
	sprintf(list->lineBuf, topline, "From", "To", "Subject");
	list->putstring(1, 3, list->lineBuf);

	if (mm.areaList->isCollection())
		list->putstring(1, COLS - 19, "Area");
	list->touch();

	Draw();
	Select();
}

void LetterListWindow::Select()
{
	mm.letterList->gotoActive(active);
}

void LetterListWindow::ResetActive()
{
        active = mm.letterList->getActive();
}

void LetterListWindow::Delete()
{
	delete list;
}

void LetterListWindow::extrakeys(int key)
{
	Select();
	switch (key) {
	case 'A':
		interface->addressbook(1);
		break;
	case 'C':
		interface->TransToggle();
		break;
	case '-':
	case KEY_LEFT:
		PrevUnread();
		break;
	case '\t':
	case '+':
	case KEY_RIGHT:
		NextUnread();
		break;
	case 'U':
	case 'M':	// Toggle read/unread and marked from letterlist
		mm.letterList->setStatus(mm.letterList->getStatus() ^
			((key == 'U') ? MS_READ : MS_MARKED));
		interface->setAnyRead();
		Move(DOWN);
		Draw();
		break;
	case 'E':
		if (mm.areaList->isReplyArea())
			interface->letterwindow.KeyHandle('E');
		else
		    if (!mm.areaList->isCollection()) {
			if (mm.areaList->isNetmail())
				interface->addressbook();
			interface->letterwindow.set_Letter_Params(
				mm.areaList->getAreaNo(), 'E');
			interface->letterwindow.EnterLetter();
		    }
		interface->changestate(letterlist);
		break;
	case 'K':
		if (mm.areaList->isReplyArea())
			interface->kill_letter();
		break;
	case 'L':
		mm.letterList->relist();
		ResetActive();
		interface->changestate(letterlist);
		break;
	case 'S':
		interface->letterwindow.Save(1);
		interface->changestate(letterlist);
	}
}
