/*
 * MultiMail offline mail reader
 * message display and editing

 Copyright (c) 1996 Kolossvary Tamas <thomas@tvnet.hu>
 Copyright (c) 1997 John Zero <john@graphisoft.hu>
 Copyright (c) 1998 William McBrine <wmcbrine@clark.net>

 Distributed under the GNU General Public License.
 For details, see the file COPYING in the parent directory. */

#include "interfac.h"
#include "isoconv.h"
#include "mysystem.h"

#include <sys/utsname.h>

Line::Line(int size)
{
	next = NULL;
	text = (size ? new char[size] : 0);
}

Line::~Line()
{
	delete[] text;
}

LetterWindow::LetterWindow()
{
	linelist = NULL;
	NM.zone = 0;
	To[0] = tagline1[0] = '\0';
}

net_address LetterWindow::PickNetAddr()
{
	Line *line;
	net_address result;
	int i;

	line = linelist[0];
	for (i = 0; i < NumOfLines; i++)
		if (!strncmp(" * Origin:", linelist[i]->text, 10))
			break;

	if (i != NumOfLines) {
		line = linelist[i];
		i = line->length;
		while (((line->text[i - 1]) != '(') && i > 0)
			i--;
		//we have the opening bracket

		while ((i < (int) line->length) &&
			((line->text[i] < '0') || (line->text[i] > '9')))
				i++;
		//we have the begining of the address

		result.init(&line->text[i]);
	} else
		result = *(mm.letterList->getNetAddr());

	return result;
}

void LetterWindow::ReDraw()
{
	headbar->wtouch();
	header->wtouch();
	text->wtouch();
	statbar->wtouch();
}

void LetterWindow::DestroyChain()
{
	if (linelist) {
		while (NumOfLines)
			delete linelist[--NumOfLines];
		delete[] linelist;
		linelist = NULL;
	}
	letter_in_chain = -1;
}

void LetterWindow::MakeChain(int columns)
{
	Line head(0), *curr;
	char *message, *j, *k, *begin;
	int c, end = 0;

	DestroyChain();
	message = (char *) mm.letterList->getBody();
	charconv_in(message);
	letter_in_chain = mm.letterList->getCurrent();
	curr = &head;

	j = message;
	while (!end) {
		curr->next = new Line(columns + 1);
		curr = curr->next;

		NumOfLines++;

		begin = NULL;
		k = curr->text;
		c = 0;

		while ((*j != 10) && !end && (c < columns)) {
			if (*j == ' ')
				begin = NULL;
			else
				if (!begin)
					begin = j;
			if (*j) {
				*k++ = *j++;
				c++;
			} else
				end = 1;
		}
		if ((c == columns) && begin && ((j - begin) < columns)) {
			c -= j - begin;
			j = begin;
		}
		if (*j == 10)
			j++;

		curr->text[c] = '\0';
		curr->length = c;
	}

	linelist = new Line *[NumOfLines];
	curr = head.next;
	c = 0;
	while (curr) {
		linelist[c++] = curr;
		curr = curr->next;
	}
}

void LetterWindow::StatToggle(int mask)
{
	int stat = mm.letterList->getStatus();
	stat ^= mask;
	mm.letterList->setStatus(stat);
	DrawHeader();
}

void LetterWindow::Draw()
{
	mm.letterList->setRead();	// nem ide kene? de.
	DrawHeader();
	DrawBody();
}

void LetterWindow::netAdd(char *tmp)
{
	net_address na = *(mm.letterList->getNetAddr());
	if (na.zone) {
		strcat(tmp, " @ ");
		strcat(tmp, na.text());
	}
}

#define headerput1(x, y) header->putstring(x, 2, y)
#define headerput2(x, y) header->putstring(x, COLS - 33, y)
#define flagattr(x) header->attrib((x) ? C_LHFLAGSHI : C_LHFLAGS)

void LetterWindow::DrawHeader()
{
	char tmp[256];
	int stat = mm.letterList->getStatus();

	header->Clear(C_LHEADTEXT);

	headerput1(0, "Msg#:");
	headerput1(1, "From:");
	headerput1(2, "  To:");
	headerput1(3, "Subj:");

	headerput2(0, "Date:");
	headerput2(1, "Line:");
	headerput2(2, "Stat:");

	flagattr(mm.letterList->getPrivate());
	header->putstring(2, COLS - 27, "Pvt");
	flagattr(stat & MS_READ);
	header->putstring(2, COLS - 23, "Read");
	flagattr(stat & MS_REPLIED);
	header->putstring(2, COLS - 18, "Replied");
	flagattr(stat & MS_MARKED);
	header->putstring(2, COLS - 10, "Marked");

	header->attrib(C_LHMSGNUM);
	sprintf(tmp, "%d (%d of %d)   ", mm.letterList->getMsgNum(),
		mm.letterList->getCurrent() + 1, mm.areaList->getNoOfLetters());
	header->putstring(0, 8, tmp);

	header->attrib(C_LHFROM);
	strcpy(tmp, mm.letterList->getFrom());
	charconv_in(tmp);

	if (mm.areaList->isNetmail())
		netAdd(tmp);

	header->putstring(1, 8, tmp);

	header->attrib(C_LHTO);
	strcpy(tmp, mm.letterList->getTo());
	charconv_in(tmp);

	if (mm.areaList->isReplyArea())
		netAdd(tmp);

	header->putstring(2, 8, tmp);

	header->attrib(C_LHSUBJ);
	sprintf(tmp, "%-71.71s", mm.letterList->getSubject());
	charconv_in(tmp);
	header->putstring(3, 8, tmp);

	for (int i = strlen(tmp) + 8; i < COLS; i++)
		header->putch(3, i, ' ');

	header->attrib(C_LHDATE);
	strcpy(tmp, mm.letterList->getDate());
	charconv_in(tmp);
	header->putstring(0, COLS - 27, tmp);

	//header->putch(4, 0, ACS_LLCORNER);
	//header->horizline(4, COLS-2);
	//header->putch(4, COLS-1, ACS_LRCORNER);

	lineCount();
}

void LetterWindow::lineCount()
{
	char tmp[80];

	header->attrib(C_LHMSGNUM);
	sprintf(tmp, "%5d/%-21d", position + 1, NumOfLines);
	header->putstring(1, COLS-27, tmp);
	header->delay_update();
}

void LetterWindow::oneLine(int i)
{
	char test, *t, *r, *currtext;
	int j, length, z = position + i;

	if (z < NumOfLines) {
		currtext = linelist[z]->text;
		length = (int) linelist[z]->length;

		text->attrib(C_LTEXT);
		for (j = 0; j < 5; j++)
			if (currtext[j] == '>') {	//quoting
				text->attrib(C_LQTEXT);
				break;
			}

		test = currtext[0];
		if ((currtext[1] == test) && (currtext[2] == test) &&
		   ((currtext[3] == ' ') || length == 3))
			switch (test) {
			case '.':
				text->attrib(C_LTAGLINE); //tagline
				t = &currtext[4];
				r = tagline1;
				if (!*r) {
					for (j = 0; (*t) && (*t != '\r') &&
					     (*t != '\n') && j < 79;
						t++, r++, j++)
						   *r = *t;
					*r = 0;
				}
				break;
			case '-':
			case '~':
				text->attrib(C_LTEAR);	//mailer
			}
		else if (!strncmp(currtext, " * Origin:", 10))
			text->attrib(C_LORIGIN);	//origin line

		text->putstring(i, 0, currtext);
	} else
		length = 0;

	for (j = length; j < COLS; j++)
		text->putch(i, j, ' ');
}

void LetterWindow::DrawBody()
{
	if (letter_in_chain != mm.letterList->getCurrent())
		MakeChain(COLS);

	lineCount();

	for (int i = 0; i < y; i++)
		oneLine(i);
	text->delay_update();
}

void LetterWindow::MakeActive(int redo)
{
	DestroyChain();

	y = LINES - 7;

	headbar = new Win(1, COLS, 0, 0, C_LBOTTSTAT);
	header = new Win(5, COLS, 1, 0, C_LHEADTEXT);
	text = new Win(y, COLS, 6, 0, C_LTEXT);
	statbar = new Win(1, COLS, LINES - 1, 0, C_LBOTTSTAT);

	statbar->cursor_on();

	char *tmp = new char[COLS + 1];
	sprintf(tmp, " " MM_TOPHEADER, MM_NAME, MM_MAJOR, MM_MINOR);
	int i;
	for (i = strlen(tmp); i < COLS; i++)
		tmp[i] = ' ';
	tmp[i] = '\0';

	headbar->putstring(0, 0, tmp);
	headbar->delay_update();

	static const char *helpmsg = "F1 or ? - Help ";
	char format[20];
	int maxw = COLS - 16;
	sprintf(format, " %%-%d.%ds%%s", maxw, maxw);
	sprintf(tmp, format, mm.areaList->getDescription(), helpmsg);
	charconv_in(tmp);

	statbar->putstring(0, 0, tmp);
	delete[] tmp;

	statbar->delay_update();
	statbar->cursor_off();

	linelist = NULL;
	NumOfLines = 0;
	if (redo) {
		position = 0;
		tagline1[0] = '\0';
	}
	Draw();
}

void LetterWindow::Next()
{
	if (mm.letterList->getActive() < (mm.letterList->noOfActive() - 1)) {
		interface->letters.Move(DOWN);
		mm.letterList->gotoActive(mm.letterList->getActive() + 1);
		position = 0;
		tagline1[0] = '\0';
		Draw();
	} else {
		interface->back();
		doupdate();
		interface->back();
		doupdate();
		interface->areas.KeyHandle(KEY_RIGHT);
	}
}

void LetterWindow::Previous()
{
	if (mm.letterList->getActive() > 0) {
		interface->letters.Move(UP);
		mm.letterList->gotoActive(mm.letterList->getActive() - 1);
		position = 0;
		tagline1[0] = '\0';
		Draw();
	} else {
		interface->back();
		doupdate();
		interface->back();
		doupdate();
		interface->areas.Move(UP);
	}
}

void LetterWindow::Move(int dir)
{
	switch (dir) {
	case KEY_UP:
		if (position > 0) {
			position--;
			text->wscroll(-1);
			oneLine(0);
			text->delay_update();
			lineCount();
		}
		break;
	case KEY_DOWN:
		if (position < NumOfLines - y) {
			position++;
			lineCount();
			text->wscroll(1);
			oneLine(y - 1);
			text->delay_update();
		}
		break;
	case KEY_HOME:
		position = 0;
		DrawBody();
		break;
	case KEY_END:
		if (NumOfLines > y) {
			position = NumOfLines - y;
			DrawBody();
		}
		break;
	case 'B':
	case KEY_PPAGE:
		position -= ((y < position) ? y : position);
		DrawBody();
		break;
	case 'F':
	case KEY_NPAGE:
		if (position < NumOfLines - y) {
			position += y;
			if (position > NumOfLines - y)
				position = NumOfLines - y;
			DrawBody();
		}
		break;
	}
}

void LetterWindow::NextDown()
{
	position += y;
	if (position >= NumOfLines)
		Next();
	else
		DrawBody();
}

void LetterWindow::Delete()
{
	DestroyChain();
	delete headbar;
	delete header;
	delete text;
	delete statbar;
}

void LetterWindow::Save(int all)
{
	FILE *fd;
	char filename[255];

	ShadowedWin question(5, 60, 10, 10, C_LLSAVETEXT);
	question.putstring(1, 2, "Save to file:");
	question.putstring(2, 2, "<");
	question.putstring(2, 57, ">");
	question.putstring(3, 30, "<ESC> (pause) to cancel");

	if (all)
		sprintf(filename, "%.8s.all", mm.areaList->getName());
	else
		sprintf(filename, "%.8s.%.03d", mm.areaList->getName(),
			mm.letterList->getCurrent());

	for (unsigned c = 0; c < strlen(filename); c++)
		if (filename[c] == ' ')
			filename[c] = '_';

	if (question.getstring(2, 3, filename, 54, C_LLSAVETEXT, C_LLSAVEGET))
	{
		mychdir(mm.resourceObject->get(SaveDir));
		if ((fd = fopen(filename, "at"))) {
			if (all)
				for (int i = 0; i <
				   mm.areaList->getNoOfLetters(); i++) {
					mm.letterList->gotoLetter(i);
					write_to_file(fd);
				}
			else
				write_to_file(fd);
			fclose(fd);
			if (!all)
				MakeChain(COLS);
		}
	}
}

void LetterWindow::set_Letter_Params(net_address *nm, const char *to)
{
	NM = *nm;
	strncpy(To, to ? to : "", 29);
}

void LetterWindow::set_Letter_Params(int area, char param_key)
{
	key = param_key;
	if (key == 'N') {
		int i = mm.areaList->findNetmail();
		replyto_area = (i != -1) ? i : area;
	} else
		replyto_area = area;
}

void LetterWindow::QuoteText(const char *reply_filename)
{
	FILE *reply;
	char TMP[81], mg[4];
	int end, i;

	const char *TMP2 = mm.letterList->getFrom();

	sprintf(TMP, "-=> %.30s wrote to %.30s <=-\n\n",
		TMP2, mm.letterList->getTo());
	charconv_in(TMP);

	strncpy(mg, TMP2, 2);
	mg[2] = '\0';
	mg[3] = '\0';

	i = 1;
	for (int j = 1; j < 3; j++) {
		end = 0;
		while (TMP2[i] && !end) {
			if ((TMP2[i - 1] == ' ') && (TMP2[i] != ' ')) {
				mg[j] = TMP2[i];
				if (j == 1)
					end = 1;
			}
			i++;
		}
	}
	charconv_in(mg);

	MakeChain(75);

	if ((reply = fopen(reply_filename, "wt"))) {
		fputs(TMP, reply);
		for (i = 0; i < NumOfLines; i++)
			fprintf(reply, " %s> %s\n", mg, linelist[i]->text);
		fclose(reply);
	}

	MakeChain(COLS);
}

int LetterWindow::HeaderLine(ShadowedWin *X, char *Y, int limit, int pos,
			int color)
{
	charconv_in(Y);
	int getit = X->getstring(pos, 8, Y, limit, color, color);
	charconv_out(Y);
	return getit;
}

int LetterWindow::EnterHeader(char *FROM, char *TO, char *SUBJ, int *privat)
{
	ShadowedWin *rep_header;
	char NETADD[25];
	int result, end = 0, current = 0, maxitems = 3;

	if (mm.areaList->isNetmail() || NM.zone) {
		strcpy(NETADD, NM.text());
		maxitems++;
	}

	rep_header = new ShadowedWin(maxitems + 2, COLS - 2, (LINES / 2) - 3,
		1, C_LETEXT);

	rep_header->putstring(1, 2, "From:");
	rep_header->putstring(2, 2, "  To:");
	if (maxitems == 4)
		rep_header->putstring(3, 2, "Addr:");
	rep_header->putstring(maxitems, 2, "Subj:");
	rep_header->update();

	do {
		result = HeaderLine(rep_header, (current == (maxitems - 1)) ?
			SUBJ : (((current == 2) && (maxitems == 4)) ? 
			NETADD : (current ? TO : FROM)),
			(current == (maxitems - 1)) ? 69 : 25,
			current + 1, (current & 1) ? C_LEGET1 : C_LEGET2);

		switch (result) {
		case 0:
			end = 1;
			break;
		case 1:
			current++;
			if (current == maxitems)
				end = 1;
			break;
		case 2:
			if (current > 0)
				current--;
			break;
		case 3:
			if (current < maxitems - 1)
				current++;
		}
	} while (!end);

	if (result) {
		if (mm.areaList->isNetmail() || NM.zone)
			NM.init(NETADD);
		if (!interface->WarningWindow(
			"Make letter private?", *privat ? "Yes" : "No",
				*privat ? "No" : "Yes"))
					*privat = !*privat;
	}

	delete rep_header;

	return result;
}

void LetterWindow::editletter(const char *reply_filename)
{
	char command[256];

	sprintf(command, "%s %s", mm.resourceObject->get(editor),
		canonize(reply_filename));
	mysystem(command);
}

long LetterWindow::reconvert(const char *reply_filename)
{
	FILE *reply;

	reply = fopen(reply_filename, "rt");
	fseek(reply, 0, SEEK_END);
	long replen = ftell(reply);
	rewind(reply);

	char *body = new char[replen + 1];
	replen = (long) fread(body, 1, replen, reply);
	fclose(reply);

	body[replen] = '\0';
	charconv_out(body);
	while (body[replen - 1] == '\n')
		replen--;

	reply = fopen(reply_filename, "wt");
	fwrite(body, 1, replen, reply);
	fclose(reply);

	delete[] body;
	return replen;
}

void LetterWindow::EnterLetter()
{
	FILE *reply;
	char reply_filename[256], FROM[26], TO[26], SUBJ[78];
	long replen;
	int privat, replyto_num;

	mm.areaList->gotoArea(replyto_area);

	// HEADER

	int usealias = mm.areaList->getUseAlias();
	if (usealias) {
		sprintf(FROM, "%.25s", mm.resourceObject->get(AliasName));
		if (!FROM[0])
			usealias = 0;
	}
	if (!usealias)
		sprintf(FROM, "%.25s", mm.resourceObject->get(LoginName));

	if (key == 'E') {
		strcpy(TO, (To[0] ? To : "All"));
		SUBJ[0] = '\0';	//we don't have subject yet
	} else {
		sprintf(TO, "%.25s", (key == 'O') ? mm.letterList->getTo() :
			mm.letterList->getFrom());
		sprintf(SUBJ, "Re: %.65s", mm.letterList->getSubject());
	}

	if ((key == 'N') || mm.areaList->isNetmail())
		privat = 1;
	else
		privat = (key == 'E') ? 0 : mm.letterList->getPrivate();

	interface->areas.Select();

	if (!EnterHeader(FROM, TO, SUBJ, &privat))
		return;

	replyto_num = (key == 'E') ? 0 : mm.letterList->getMsgNum();

	// BODY

	mytmpnam(reply_filename);

	// Quote the old text, mark original as replied
	if (key != 'E') {
		QuoteText(reply_filename);
		mm.letterList->setStatus(mm.letterList->getStatus() |
			MS_REPLIED);
	}

	// Edit the reply
	editletter(reply_filename);

	reply = fopen(reply_filename, "at");

	// Signature
	const char *sg = mm.resourceObject->get(sigFile);
	if (sg) {
		FILE *s = fopen(sg, "rt");
		if (s) {
			char bufx[81];

			fputc('\n', reply);
			while (fgets(bufx, 79, s))
				fputs(bufx, reply);
			fclose(s);
		}
	}

	// Tagline
	tagline1[0] = '\0';

	interface->Tagwin();
	interface->KeyHandle();

	if (tagline1[0])
		fprintf(reply, "\n... %s\n", tagline1);

	// Tearline (not for Blue Wave -- it does its own)
	if (mm.driverList->useTearline()) {
		struct utsname buf;
		uname(&buf);
		if (!tagline1[0])
			fputc('\n', reply);
		fprintf(reply, "--- %s/%s v%d.%d\n", MM_NAME, buf.sysname,
			MM_MAJOR, MM_MINOR);
	}

	fclose(reply);

	tagline1[0] = '\0';

	// Reconvert the text
	replen = reconvert(reply_filename);

	mm.areaList->enterLetter(replyto_area, FROM, TO, SUBJ, replyto_num,
		privat, &NM, reply_filename, (int) replen);

	NM.zone = 0;
	To[0] = '\0';

	interface->areas.Select();
	interface->setUnsaved();
}

void LetterWindow::EditLetter()
{
	FILE *reply;
	char reply_filename[256], FROM[26], TO[26], SUBJ[78], *body;
	long siz;
	int privat, replyto_num, replyto_area;

	strcpy(FROM, mm.letterList->getFrom());
	strcpy(TO, mm.letterList->getTo());
	strcpy(SUBJ, mm.letterList->getSubject());
	privat = mm.letterList->getPrivate();

	NM = *(mm.letterList->getNetAddr());

	if (!EnterHeader(FROM, TO, SUBJ, &privat))
		return;

	DestroyChain();		// current letter's chain reset

	replyto_num = mm.letterList->getReplyTo();
	replyto_area = mm.letterList->getAreaID();

	mytmpnam(reply_filename);

	body = (char *) mm.letterList->getBody();
	charconv_in(body);
	reply = fopen(reply_filename, "wt");
	fwrite(body, strlen(body), 1, reply);
	fclose(reply);
	body = NULL;		// it will be auto-dealloc'd by next getBody

	editletter(reply_filename);
	siz = reconvert(reply_filename);

	mm.areaList->killLetter(mm.letterList->getMsgNum());
	mm.areaList->enterLetter(replyto_area, FROM, TO, SUBJ, replyto_num,
		privat, &NM, reply_filename, (int) siz);

	mm.letterList->rrefresh();
	interface->areas.Select();
	if (interface->active() == letter)
		interface->back();

	NM.zone = 0;

	interface->setUnsaved();
}

void LetterWindow::set_Tagline(const char *tl)
{
	strncpy(tagline1, tl, 76);
}

void LetterWindow::GetTagline()
{
	interface->taglines.EnterTagline(tagline1);
	ReDraw();
}

void LetterWindow::write_to_file(FILE *fd)
{
	char Header[400];
	int j;

	//write header to file
	for (j = 0; j < 80; j++)
		fputc('=', fd);

	sprintf(Header,
		"\n System: %.71s\n   Area: %.71s\n   Date: %.25s\n"
		"   From: %.45s\n     To: %.45s\n   Subj: %.71s\n",
		mm.resourceObject->get(BBSName),
		mm.areaList->getDescription(), mm.letterList->getDate(),
		mm.letterList->getFrom(), mm.letterList->getTo(),
		mm.letterList->getSubject());

	charconv_in(Header);
	fputs(Header, fd);

	for (j = 0; j < 80; j++)
		fputc('-', fd);
	fputc('\n', fd);

	//write chain to file
	MakeChain(80);
	for (j = 0; j < NumOfLines; j++)
		fprintf(fd, "%s\n", linelist[j]->text);
	fputc('\n', fd);
}

void LetterWindow::KeyHandle(int key)
{
	switch (key) {
	case 'A':
		interface->addressbook(1);
		break;
	case 'C':
		interface->TransToggle();
		break;
	case 'S':
		Save(0);
		ReDraw();
		break;
	case '?':
	case KEY_F(1):
		interface->changestate(letter_help);
		break;
	case 'V':
	case 1: // CTRL-A
	case 11: // CTRL-V
		interface->ansiLoop(mm.letterList->getBody(),
			mm.letterList->getSubject());
		break;
	case '+':
	case KEY_RIGHT:
		Next();
		break;
	case '-':
	case KEY_LEFT:
		Previous();
		break;
	case ' ':
		NextDown();
		break;
	default:
	    if (mm.areaList->isReplyArea()) {
		switch(key) {
		case 'E':
			EditLetter();
			break;
		case 'K':
			interface->kill_letter();
			break;
		default:
			Move(key);
		}
	    } else {
		switch (key) {
		case 'M':
		case 'U':
			StatToggle((key == 'M') ? MS_MARKED : MS_READ);
			break;
		case 'R':
		case 'O':
		case 'E':
			interface->changestate(littlearealist);
			break;
		case 'N':
			if (mm.areaList->findNetmail() != -1) {
			    net_address nm = PickNetAddr();
			    if (nm.zone) {
				set_Letter_Params(
					mm.areaList->getAreaNo(), 'N');
				set_Letter_Params(&nm, NULL);
				EnterLetter();
				DrawHeader();
				ReDraw();
			    }
			}
			break;
		case 'T':
			GetTagline();
			break;
		default:
			Move(key);
		}
	    }
	}
}
