/*
 * MultiMail offline mail reader
 * packet list window, vanity plate

 Copyright (c) 1996 Kolossvary Tamas <thomas@vma.bme.hu>
 Copyright (c) 1997 John Zero <john@graphisoft.hu>
 Copyright (c) 1998 William McBrine <wmcbrine@clark.net>

 Distributed under the GNU General Public License.
 For details, see the file COPYING in the parent directory. */

#include "interfac.h"
#include "mysystem.h"

void Welcome::MakeActive()
{
	window = new ShadowedWin(7, 58, 2, COLS / 2 - 29, C_WELCOME1 |
		A_BOLD);
	window->putstring(1, 8,
		"Welcome to " MM_NAME " Blue Wave/QWK reader!");
	window->attrib(C_WELCOME2 | A_BOLD);
	window->putstring(3, 2,
		"maintained by:    William McBrine <wmcbrine@clark.net>");
	window->putstring(4, 2,
		"original authors: Kolossvary Tamas, Toth Istvan, and");
	window->putstring(5, 20, "John Zero");
	window->touch();
}

void Welcome::Delete()
{
	delete window;
}

PacketListWindow::PacketListWindow()
{
	char tmp[512];

	packetList = new file_list(mm.resourceObject->get(PacketDir));

	if (!NumOfItems()) {
		sprintf(tmp, "No packets found.\n\n"
			"Please place any packets to be read in:\n%s",
			mm.resourceObject->get(PacketDir));
		fatalError(tmp);
	}
}

PacketListWindow::~PacketListWindow()
{
	delete packetList;
}

void PacketListWindow::MakeActive()
{
	welcome.MakeActive();
	oldPos = oldActive = -100;
	list_max_y = LINES - 19;
	list = new InfoWin(LINES - 16, 60, 10, COLS / 2 - 30, C_PBBACK);
	list->attrib(C_PHEADTEXT);
	list->putstring(1, 3, "Packet                Size    Date");
	list->touch();
	Draw();
}

int PacketListWindow::NumOfItems()
{
	return packetList->getNoOfFiles();
}

void PacketListWindow::Delete()
{
	delete list;
	welcome.Delete();
}

void PacketListWindow::oneLine(int i)
{
	char *tmp = list->lineBuf;

	packetList->gotoFile(position + i);

	const char *tmp2 = packetList->getName();
	time_t tmpt = packetList->getDate();

	strcpy(tmp, "          ");

	for (int j = 2; (*tmp2 != '.') && (j < 10); j++)
		tmp[j] = *tmp2++;

	sprintf(&tmp[10], "%-4.4s%16u  %s", tmp2,
		(unsigned) packetList->getSize(), ctime(&tmpt));

	strcpy(&tmp[56], "  ");

	list->oneline(i, (position + i == active) ? A_STANDOUT : C_PLINES);
}

void PacketListWindow::Select()
{
	packetList->gotoFile(active);
}

void PacketListWindow::ReList()
{
	if (position > 1) {
		if (active > 1)
			active--;
		position--;
	}
	delete packetList;
	packetList = new file_list(mm.resourceObject->get(PacketDir));
}

void PacketListWindow::extrakeys(int key)
{
	switch (key) {
	case 'A':
		interface->addressbook(1);
		break;
	case 'K':
		char tmp[128];

		Select();
		sprintf(tmp, "Do you really want to delete %s?",
			packetList->getName());
		if (interface->WarningWindow(tmp, "Yes", "No")) {
			mychdir(mm.resourceObject->get(PacketDir));
			remove(packetList->getName());
			ReList();
		}
		interface->changestate(packetlist);
		break;
	case '-':
	case KEY_LEFT:
		Move(UP);
		Draw();
		break;
	case '+':
	case KEY_RIGHT:
		Move(DOWN);
		Draw();
	}
}

void PacketListWindow::OpenPacket()
{
	Select();
	mm.selectPacket(packetList->getName());
}
