/*
 * MultiMail offline mail reader
 * tagline selection, editing

 Copyright (c) 1996 Kolossvary Tamas <thomas@vma.bme.hu>
 Copyright (c) 1998 William McBrine <wmcbrine@clark.net>

 Distributed under the GNU General Public License.
 For details, see the file COPYING in the parent directory. */

#include "interfac.h"

tagline::tagline()
{
	next = NULL;
}

TaglineWindow::TaglineWindow()
{
	nodraw = 1;
	ReadFile();
}

TaglineWindow::~TaglineWindow()
{
	DestroyChain();
}

void TaglineWindow::MakeActive()
{
	nodraw = 0;
	oldActive = oldPos = -100;

	srand((unsigned) time(NULL));

	list_max_y = LINES - 15;
	list = new InfoWin(LINES - 10, COLS - 2, 5, 1,
		C_TBBACK, "Select tagline", C_TTEXT, 5, 1);

	list->attrib(C_TTEXT);
	list->putstring(LINES - 13, 3, " - Don't apply tagline");
	list->putstring(LINES - 13, 28, " - Random select tagline");
	list->putstring(LINES - 13, 59, " - Apply tagline");
	list->putstring(LINES - 12, 3, " - Reload tagline file");
	list->putstring(LINES - 12, 28, " - Manual enter tagline");
	list->attrib(C_TKEYSTEXT);
	list->horizline(LINES - 14, COLS - 4);
	list->putstring(LINES - 13, 2, "Q");
	list->putstring(LINES - 13, 27, "R");
	list->putstring(LINES - 13, 54, "Enter");
	list->putstring(LINES - 12, 2, "L");
	list->putstring(LINES - 12, 27, "E");
	Draw();
}

void TaglineWindow::Delete()
{
	delete list;
	nodraw = 1;
}

void TaglineWindow::extrakeys(int key)
{
	switch (key) {
	case '\r':
	case '\n':
		if (NumOfTaglines > 0)
			interface->letterwindow.set_Tagline(
				getTextOf(active));
		break;
	case 'E':
		EnterTagline();
		break;
	case 'L':
		ReadFile();
		break;
	case 'R':
		RandomTagline();
		break;
	case '+':
	case KEY_RIGHT:
		Move(DOWN);
		Draw();
		break;
	case '-':
	case KEY_LEFT:
		Move(UP);
		Draw();
	}
}

void TaglineWindow::RandomTagline()
{
	int i = rand() / (RAND_MAX / NumOfTaglines);

	Move(HOME);
	for (int j = 1; j <= i; j++)
		Move(DOWN);
	Draw();
}

void TaglineWindow::EnterTagline(const char *tag)
{
	FILE *fd;
	char newtagline[TAGLINE_LENGTH + 1];
	int y;

	Move(END);
	if (NumOfTaglines >= list_max_y) {
		y = list_max_y;
		position++;
	} else
		y = NumOfTaglines + 1;
	active++;

	if (!nodraw)
		Draw();
	else {
		list = new InfoWin(5, COLS - 2, 10, 1, C_TBBACK);
		list->attrib(C_TTEXT);
		list->putstring(1, 1, "Enter new tagline:");
		list->update();
	}

	strcpy(newtagline, tag ? tag : "");

	if (list->getstring((nodraw) ? 2 : y, 1, newtagline,
		TAGLINE_LENGTH, C_TENTER, C_TENTERGET)) {

		char *p;
		for (p = &newtagline[strlen(newtagline) - 1];
		     (p > newtagline) && (*p == ' '); p--);	//crop spaces

		*++p = '\0';

		if (newtagline[0]) {
			tagline *curr = &head;
			//check dupes
			while (curr && strcmp(newtagline, curr->text))
				curr = curr->next;
			if (!curr)
				if ((fd = fopen(mm.resourceObject->
				get(TaglineFile), "at"))) {
					fputs(newtagline, fd);
					fputc('\n', fd);
					fclose(fd);
				}
			ReadFile();
		}
	}
	Move(END);

	if (!nodraw) {
		Draw();
		doupdate();
	} else
		list->update();
}

void TaglineWindow::ReadFile()
{
	FILE *fd;

	if ((fd = fopen(mm.resourceObject->get(TaglineFile), "rt"))) {
		DestroyChain();
		NumOfTaglines = position = active = 0;

		tagline *curr = &head;
		while (fgets(curr->text, TAGLINE_LENGTH, fd)) {
			if (curr->text[strlen(curr->text) - 1] == '\n')
				curr->text[strlen(curr->text) - 1] = '\0';
			curr->next = new tagline;
			curr = curr->next;
			NumOfTaglines++;
		}
		fclose(fd);
	}
}

void TaglineWindow::DestroyChain()
{
	tagline *curr, *tmp;

	curr = head.next;
	while (curr) {
		tmp = curr;
		curr = curr->next;
		delete tmp;
	}
	head.next = NULL;
}

void TaglineWindow::oneLine(int i)
{
	sprintf(list->lineBuf, "%-76.76s", (position + i > NumOfTaglines) ?
		" " : getTextOf(position + i));

	list->oneline(i,
		C_TLINES | ((position + i == active) ? A_REVERSE : 0));
}

int TaglineWindow::NumOfItems()
{
	return NumOfTaglines;
}

const char *TaglineWindow::getTextOf(int num)
{
	tagline *curr = &head;

	for (int i = 1; (i <= num) && (i < NumOfTaglines); i++)
		curr = curr->next;
	return curr->text;
}
