/*
 * MultiMail offline mail reader
 * mmail class

 Copyright (c) 1996 Toth Istvan <stoty@vma.bme.hu>
 Copyright (c) 1998 William McBrine <wmcbrine@clark.net>

 Distributed under the GNU General Public License.
 For details, see the file COPYING in the parent directory. */

#ifndef MMAIL_H 
#define MMAIL_H

#include "misc.h"
#include "resource.h"

#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <sys/types.h>

// Number of the Reply area
#define REPLY_AREA 0

// Area types
#define COLLECTION 1
#define REPLYAREA 2
#define ACTIVE 4
#define ALIAS 8
#define NETMAIL 16
#define INTERNET 32

// Mail statuses -- bit f.
#define MS_NONE		0
#define MS_READ		1
#define MS_REPLIED	2 
#define MS_MARKED	4

// for letter_list::sort
#define LS_SUBJ     1
#define LS_NAME     2 // not implemented yet
#define LS_MSGNUM   3 // -||- 

// Some of the functions normally used by MultiMail don't exist in EMX,
// but are available under other names:

#ifdef __EMX__
# define strcasecmp stricmp
# define strncasecmp strnicmp
extern "C" {
int _chdir2(__const__ char *);
char *_getcwd2(char *, int);
}
# define mychdir _chdir2
# define mygetcwd _getcwd2
#else
# define mychdir chdir
# define mygetcwd getcwd
#endif

class mmail;
class resource;
class file_header;
class file_list;
class area_header;
class area_list;
class letter_header;
class letter_list;
class specific_driver;
class reply_driver;
class driver_list;
class read_class;

class net_address
{
 public:
	int zone, net, node, point;

	const char *text();
	void init(const char *);
};
 
class mmail
{
 public:
	resource *resourceObject;
	file_list *workList;
	driver_list *driverList;
	area_list *areaList;
	letter_list *letterList;

	mmail();
	~mmail();
	void selectPacket(const char *);
	void Delete();
	void saveRead();
};	

class file_header
{
	char *name;
	time_t date;
	off_t size;
 public:
	file_header *next;

	file_header(char *, time_t, off_t);
	~file_header();
	inline const char *getName() const;
	inline time_t getDate() const;
	inline off_t getSize() const;
};

class file_list
{
	file_header **files;
	int noOfFiles, activeFile;

	void initFiles(const char *);
	void fileqsort(int, int);
 public:
	file_list(const char *);
	~file_list();
	int getNoOfFiles() const;
	void gotoFile(int);
	const char *getName() const;
	const char *exists(const char *);
	const char *getNext(const char *);
	FILE *ftryopen(const char *, const char *);
	time_t getDate() const;
	off_t getSize() const;
};

class area_header
{
	mmail *mm;
	specific_driver *driver;
	const char *shortName, *name, *description, *areaType;
	int noOfLetters, noOfPersonal, num, type;
 public:
	area_header(mmail *, int, const char *, const char *, const char *,
		const char *, int, int, int);
	inline const char *getShortName() const;
	inline const char *getName() const;
	inline const char *getDescription() const;
	inline const char *getAreaType() const;
 	inline int getNoOfLetters() const;
	inline int getNoOfUnread();
	inline int getNoOfPersonal() const;
	inline int getUseAlias() const;
	inline int isCollection() const;
	inline int isReplyArea() const;
	inline int isNetmail() const;
	int isActive() const;
};

class area_list
{
	mmail *mm;
	area_header **areaHeader;
	int no, noActive, current, shortlist, *activeHeader;

	void refreshArea();
 public:
	area_list(mmail *);
	~area_list();
	void relist();

	const char *getShortName() const;
	const char *getName() const;
	const char *getName(int);
	const char *getDescription() const;
	const char *getDescription(int);
	const char *getAreaType() const;
	int getNoOfLetters() const;
	int getNoOfUnread() const;
	int getNoOfPersonal() const;
	int getUseAlias() const;
	int isCollection() const;
	int isReplyArea() const;
	int isNetmail() const;

	void getLetterList();
	void enterLetter(int, char *, char *, char *, int, int,
			net_address *, char *, int);
	void killLetter(int);
	void makeReply();
	void gotoArea(int);
	void gotoActive(int);
	int getAreaNo() const;
	int getActive();
	int noOfAreas() const;
	int noOfActive() const;
	int findNetmail() const;
};

class letter_header
{
	driver_list *dl;
	read_class *readO;
	specific_driver *driver;
	char *subject, *to, *from, *date;
	int replyTo, LetterID, AreaID, privat, length, msgNum;
	net_address netAddr;
 public:
	letter_header(mmail *, const char *, const char *, const char *,
		const char *, int, int, int, int, int, int, specific_driver *,
		net_address *);
	~letter_header();				

	const char *getSubject() const;
	const char *getTo() const;
	const char *getFrom() const;
	net_address *getNetAddr();
	int getReplyTo() const;
	int getPrivate() const;
	inline specific_driver *getDriver() const;
	inline const char *getDate() const;
	inline const char *getBody();
	inline int getLetterID() const;
	inline int getAreaID() const;
	inline int getMsgNum() const;
	inline int getLength() const;

	inline int getRead();
	inline void setRead();
	inline int getStatus();
	inline void setStatus(int);
};

class letter_list
{ 
	driver_list *dl;
	specific_driver *driver;
	read_class *readO;
	letter_header **letterHeader;
	int noOfLetters, noActive, areaNumber, currentLetter, isColl;
	int shortlist, *activeHeader;
	
	void init();
	void cleanup();
	void lettqsort(int, int);
	//void sort(int);
 public:
	letter_list(mmail *, int, int);
	~letter_list();
	void relist();

	const char *getSubject() const;
	const char *getTo() const;
	const char *getFrom() const;
	const char *getDate() const;
	const char *getBody();
	net_address *getNetAddr();
	int getReplyTo() const;
	int getMsgNum() const;
	int getAreaID() const;
	int getPrivate() const;
	int getLength() const;

	int getStatus();
	void setStatus(int);
	int getRead();
	void setRead();
	
	void rrefresh();
	int noOfLetter() const;
	int noOfActive() const;
	void gotoLetter(int);
	void gotoActive(int);
	int getCurrent() const;
	int getActive() const;
};

class driver_list
{
	struct driver_struct {
		specific_driver *driver;
		read_class *read;
		int offset;
	} driverList[2];

	int noOfDrivers, attributes;
 public:
	driver_list(mmail *);
	~driver_list();
	int getNoOfDrivers() const;
	specific_driver *getDriver(int);
	reply_driver *getReplyDriver();
	read_class *getReadObject(specific_driver *);
	int getOffset(specific_driver *);
	int hasPersonal() const;
	int useTearline() const;
};

class read_class
{
 public:
	read_class() {};
	virtual ~read_class() {};
	virtual void setRead(int, int, int) = 0;
	virtual int getRead(int, int) = 0;
	virtual void setStatus(int, int, int) = 0;
	virtual int getStatus(int, int) = 0;
	virtual int getNoOfUnread(int) = 0;
	virtual void saveAll() = 0;
};

class main_read_class : public read_class
{
	resource *ro;
	int noOfAreas, **readStore, *noOfLetters;
 public:
	main_read_class(mmail *, specific_driver *);
	~main_read_class();
	void setRead(int, int, int);
	int getRead(int, int);
	void setStatus(int, int, int);
	int getStatus(int, int);
	int getNoOfUnread(int);
	void saveAll();
	char *readFilePath(const char *);
};

class reply_read_class: public read_class
{
 public:
	reply_read_class(mmail *, specific_driver *);
	~reply_read_class();
	void setRead(int, int, int);
	int getRead(int, int);
	void setStatus(int, int, int);
	int getStatus(int, int);
	int getNoOfUnread(int);
	void saveAll();
};	

struct bodytype
{
	long pointer, msgLength;
};

class specific_driver
{
 public:
	virtual ~specific_driver() {};
	virtual int getNoOfAreas() = 0;
	virtual void resetAll() = 0;
	virtual area_header *getNextArea() = 0;
	virtual void selectArea(int) = 0;
	virtual int getNoOfLetters() = 0;
	virtual void resetLetters() = 0;
	virtual letter_header *getNextLetter() = 0;
	virtual const char *getBody(int, int) = 0;
};

class reply_driver : public specific_driver
{
 public:	
	virtual ~reply_driver() {};
	virtual void enterLetter(letter_header *, char *, int) = 0;
	virtual void killLetter(int) = 0;
	virtual area_header *refreshArea() = 0;
	virtual void makeReply() = 0;
};

#endif
